/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */

//Title:        Cliffhanger Adventure Gear
//Version:      
//Copyright:    Copyright (c) 1997 Borland International, Inc.
//Author:       Application Methods, Inc.
//Description:  Cliffhanger Adventure Gear order entry system

package borland.reference.cliffhanger;

import java.awt.*;
import java.util.*;
import borland.sql.dataset.*;
import borland.jbcl.dataset.*;
import borland.jbcl.util.*;
import borland.jbcl.control.*;
import java.sql.Date;

/**
 * DataModule1 is an implementation of the DataModule interface.
 * The DataModule is a non visual container for components, especially
 * DataBroker components (i.e. Database and DataSet components). This
 * centralizes related database components into one module, allowing a
 * separation of business rule logic and application logic. A big advantage
 * DataModules allow for easier reuse or sharing of components between
 * multiple Frames in an application, or between multiple applets.
 *
 * <P>
 * This class employs the Singleton pattern to ensure that the class has only
 * one instance. You can access the singleton instance by calling the
 * static method getDataModule. This method instantiates the DataModule for
 * the first caller and returns this same instance to any successive callers:
 *
 * <P>
 * <CODE><PRE>
 *   DataModule1 dm = DataModule1.getDataModule();
 * <PRE></CODE>
 *
 * <P>
 * The DataSets that are used throughout the Cliffhanger application are
 * contained and initialized here in this DataModule. Any master-detail
 * relationships between datasets are defined here too. All persistent
 * columns for the various datasets are defined and initialized here also.
 * Client-side business logic is implemented in the data module to centralize
 * a common response by datasets to validation checks and database exceptions.
 * To access a dataset contained in the data module, use the getter methods
 * for the various DataSets in the data module. These methods instantiate the
 * DataSets for the first caller and returns the same instances to any
 * successive callers, allowing mulitple frames to share the same dataset
 * cursor:
 *
 * <P>
 * <CODE><PRE>
 *   DataModule1 dm = DataModule1.getDataModule();
 *   QueryDataSet customerDataSet = dm.getCustomerDataSet();
 * </PRE></CODE>
 *
 */
public class DataModule1 implements DataModule{
  private static DataModule1 myDM;
  static final double TAXPERCENT = 0.09086;
  static final double SHIPPERCENT = 0.04;

  Database database1 = new Database();
  QueryDataSet productDataSet = new QueryDataSet();
  QueryDataSet categoryDataSet = new QueryDataSet();

  // customerDataSet and custOrderDataSet are linked
  // in a master-detail relationship
  QueryDataSet customerDataSet = new QueryDataSet();
  QueryDataSet custOrderDataSet = new QueryDataSet();

  // orderDataSet and orderItemDataSet are linked
  // in a master-detail relationship
  QueryDataSet orderDataSet = new QueryDataSet();
  QueryDataSet orderItemDataSet = new QueryDataSet();

  // Lookup datasets
  QueryDataSet payMethodDataSet = new QueryDataSet();
  QueryDataSet shipMethodDataSet = new QueryDataSet();
  QueryDataSet statusDataSet = new QueryDataSet();

  // Other datasets
  QueryDataSet nextOrderIDDataSet = new QueryDataSet();
  QueryDataSet nextCustomerIDDataSet = new QueryDataSet();
  QueryDataSet accountDataSet = new QueryDataSet();

  // Query Resolvers
  QueryResolver resOrder = new QueryResolver();
  QueryResolver resOrderItem = new QueryResolver();
  QueryResolver resCustomer = new QueryResolver();
  QueryResolver resProduct = new QueryResolver();

  // Persistent columns
  // Orders
  Column colOrdersID = new Column();
  Column colOrdersCustomerID = new Column();
  Column colOrdersOrderDate = new Column();
  Column colOrdersStatus = new Column();
  Column colOrdersShipDate = new Column();
  Column colOrdersOrderTrackNum = new Column();
  Column colOrdersCustomerPONum = new Column();
  Column colOrdersPayMethod = new Column();
  Column colOrdersCreditCardNum = new Column();
  Column colOrdersWirePaymentInstr = new Column();
  Column colOrdersShipMethod = new Column();
  Column colOrdersAmtPaid = new Column();
  Column colOrdersShipName = new Column();
  Column colOrdersShipAddr1 = new Column();
  Column colOrdersShipAddr2 = new Column();
  Column colOrdersShipCity = new Column();
  Column colOrdersShipState = new Column();
  Column colOrdersShipPostalCode = new Column();
  Column colOrdersShipCountry = new Column();
  Column colOrdersBillAddr1 = new Column();
  Column colOrdersBillAddr2 = new Column();
  Column colOrdersBillCity = new Column();
  Column colOrdersBillState = new Column();
  Column colOrdersBillPostalCode = new Column();
  Column colOrdersBillCountry = new Column();
  Column colOrdersCustomerName = new Column();
  // OrderItem
  Column colOrderItemQty = new Column();
  Column colOrderItemProductID = new Column();
  Column colOrderItemProductName = new Column();
  Column colOrderItemSalePrice = new Column();
  Column colOrderItemExtendedPrice = new Column();
  Column colOrderItemSubtotal = new Column();
  Column colOrderItemOrderID = new Column();
  Column colOrderItemTax = new Column();
  Column colOrderItemShipping = new Column();
  Column colOrderItemAmtDue = new Column();
  // Customer
  Column colCustomerID = new Column();
  Column colCustomerFirstName = new Column();
  Column colCustomerMI = new Column();
  Column colCustomerLastName = new Column();
  Column colCustomerPhone = new Column();
  Column colCustomerFax = new Column();
  Column colCustomerEmail = new Column();
  Column colCustomerAddr1 = new Column();
  Column colCustomerAddr2 = new Column();
  Column colCustomerCity = new Column();
  Column colCustomerState = new Column();
  Column colCustomerPostalCode = new Column();
  Column colCustomerCountry = new Column();
  Column colCustomerShipName = new Column();
  Column colCustomerShipAddr1 = new Column();
  Column colCustomerShipAddr2 = new Column();
  Column colCustomerShipCity = new Column();
  Column colCustomerShipState = new Column();
  Column colCustomerShipPostalCode = new Column();
  Column colCustomerShipCountry = new Column();
  // Customer Order
  Column colCustOrderID = new Column();
  Column colCustOrderCustomerID = new Column();
  Column colCustOrderOrderDate = new Column();
  Column colCustOrderStatus = new Column();
  Column colCustOrderShipDate = new Column();
  // Product
  Column colProductID = new Column();
  Column colProductIsActive = new Column();
  Column colProductName = new Column();
  Column colProductCategory = new Column();
  Column colProductBasePrice = new Column();
  Column colProductDiscountPct = new Column();
  Column colProductStockQty = new Column();
  Column colProductMinReorderQty = new Column();
  ResourceBundle res = Res.getBundle("borland.reference.cliffhanger.Res");
  Column categoryColumn = new Column();
  PopupPickListItemEditor edtOrderItemProductID = new PopupPickListItemEditor();

  public DataModule1() {
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    database1.setConnection(new borland.sql.dataset.ConnectionDescriptor("jdbc:odbc:Cliffhanger", "SYSDBA", "masterkey", true, "sun.jdbc.odbc.JdbcOdbcDriver"));
    database1.setTransactionIsolation(java.sql.Connection.TRANSACTION_READ_COMMITTED);
    productDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT * FROM Product", null, true, Load.ALL));
    categoryDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT * FROM L_ProductCategory", null, true, Load.ALL));
    categoryDataSet.addEditListener(new DataModule1_categoryDataSet_editAdapter(this));
    customerDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT * FROM Customer", null, true, Load.ALL));
    custOrderDataSet.setMasterLink(new borland.jbcl.dataset.MasterLinkDescriptor(customerDataSet, new String[] {"ID"}, new String[] {"CUSTOMERID"}, true));
    custOrderDataSet.setReadOnly(true);
    custOrderDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT ID, CUSTOMERID, ORDERDATE, STATUS, SHIPDATE FROM ORDERS WHERE CUSTOMERID = :ID", null, true, Load.ALL));
    orderDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT * FROM ORDERS", null, true, Load.ALL));
    orderDataSet.addCalcFieldsListener(new DataModule1_orderDataSet_calcFieldsAdapter(this));
    orderItemDataSet.setMasterLink(new borland.jbcl.dataset.MasterLinkDescriptor(orderDataSet, new String[] {"ID"}, new String[] {"ORDERID"}, true));
    orderItemDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT ORDERID, PRODUCTID, QTY, SALEPRICE FROM ORDERITEM WHERE ORDERID = :ID", null, true, Load.ALL));
    orderItemDataSet.addCalcFieldsListener(new DataModule1_orderItemDataSet_calcFieldsAdapter(this));
    orderItemDataSet.addCalcAggFieldsListener(new DataModule1_orderItemDataSet_calcAggFieldsAdapter(this));
    payMethodDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT * FROM L_PAYMETHOD", null, true, Load.ALL));
    shipMethodDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT * FROM L_SHIPMETHOD", null, true, Load.ALL));
    statusDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT * FROM L_STATUS", null, true, Load.ALL));
    nextOrderIDDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT NEXTID FROM spNextOrderID", null, true, Load.ALL));
    nextCustomerIDDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT NEXTID FROM spNextCustomerID", null, true, Load.ALL));
    accountDataSet.setQuery(new borland.sql.dataset.QueryDescriptor(database1, "SELECT ID, CREDITCARDNUMBER FROM ACCOUNT", null, true, Load.ALL));

    // Initialize QueryResolvers
    orderDataSet.setResolver(resOrder);
    orderDataSet.addEditListener(new DataModule1_orderDataSet_editAdapter(this));
    orderItemDataSet.setResolver(resOrderItem);
    orderItemDataSet.addEditListener(new DataModule1_orderItemDataSet_editAdapter(this));
    customerDataSet.setResolver(resCustomer);
    customerDataSet.addEditListener(new DataModule1_customerDataSet_editAdapter(this));
    productDataSet.setResolver(resProduct);
    productDataSet.addEditListener(new DataModule1_productDataSet_editAdapter(this));
    resOrder.setDatabase(database1);
    resOrder.setUpdateMode(borland.jbcl.dataset.UpdateMode.CHANGED_COLUMNS);
    resOrderItem.setDatabase(database1);
    resOrderItem.setUpdateMode(borland.jbcl.dataset.UpdateMode.CHANGED_COLUMNS);
    resCustomer.setDatabase(database1);
    resCustomer.setUpdateMode(borland.jbcl.dataset.UpdateMode.CHANGED_COLUMNS);
    resProduct.setDatabase(database1);
    resProduct.setUpdateMode(borland.jbcl.dataset.UpdateMode.CHANGED_COLUMNS);
    resProduct.addResolverListener(new DataModule1_resProduct_resolverAdapter(this));

    // Initialize Persistent Columns
    // Orders
    colOrdersID.setCaption(res.getString("DM_OrderDS_ID"));
    colOrdersID.setColumnName("ID");
    colOrdersID.setDataType(borland.jbcl.util.Variant.INT);
    colOrdersCustomerID.setCaption(res.getString("DM_OrderDS_CUSTOMERID"));
    colOrdersCustomerID.setColumnName("CUSTOMERID");
    colOrdersCustomerID.setDataType(borland.jbcl.util.Variant.INT);
    colOrdersCustomerID.addColumnChangeListener(new DataModule1_colOrdersCustomerID_columnChangeAdapter(this));
    colOrdersOrderDate.setCaption(res.getString("DM_OrderDS_ORDERDATE"));
    colOrdersOrderDate.setColumnName("ORDERDATE");
    colOrdersOrderDate.setDisplayMask(res.getString("DM_DateDisplayMask"));
    colOrdersOrderDate.setDataType(borland.jbcl.util.Variant.TIMESTAMP);
    colOrdersOrderDate.setEditMask(res.getString("DM_DateEditMask"));
    colOrdersOrderDate.setDefault("NOW");
    colOrdersStatus.setCaption(res.getString("DM_OrderDS_STATUS"));
    colOrdersStatus.setColumnName("STATUS");
    colOrdersStatus.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersStatus.setDefault(res.getString("DM_Open"));
    colOrdersStatus.addColumnChangeListener(new DataModule1_colOrdersStatus_columnChangeAdapter(this));
    colOrdersShipDate.setCaption(res.getString("DM_OrderDS_SHIPDATE"));
    colOrdersShipDate.setColumnName("SHIPDATE");
    colOrdersShipDate.setDisplayMask(res.getString("DM_DateDisplayMask"));
    colOrdersShipDate.setDataType(borland.jbcl.util.Variant.TIMESTAMP);
    colOrdersShipDate.setEditMask(res.getString("DM_DateEditMask"));
    colOrdersOrderTrackNum.setCaption(res.getString("DM_OrderDS_ORDERTRACKNUM"));
    colOrdersOrderTrackNum.setColumnName("ORDERTRACKNUM");
    colOrdersOrderTrackNum.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersCustomerPONum.setCaption(res.getString("DM_OrderDS_CUSTOMERPONUM"));
    colOrdersCustomerPONum.setColumnName("CUSTOMERPONUM");
    colOrdersCustomerPONum.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersPayMethod.setColumnName("PAYMETHOD");
    colOrdersPayMethod.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersPayMethod.setDefault(res.getString("DM_VISA"));
    colOrdersPayMethod.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersPayMethod.addColumnChangeListener(new DataModule1_colOrdersPayMethod_columnChangeAdapter(this));
    colOrdersCreditCardNum.setColumnName("CREDITCARDNUM");
    colOrdersCreditCardNum.setDisplayMask("####\\-####\\-####\\-####;0; ; ");
    colOrdersCreditCardNum.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersCreditCardNum.setEditMask("####\\-####\\-####\\-####;0; ; ");
    colOrdersCreditCardNum.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersWirePaymentInstr.setColumnName("WIREPAYMENTINSTR");
    colOrdersWirePaymentInstr.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersWirePaymentInstr.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersShipMethod.setColumnName("SHIPMETHOD");
    colOrdersShipMethod.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersShipMethod.setDefault(res.getString("DM_UPS"));
    colOrdersShipMethod.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersAmtPaid.setCurrency(true);
    colOrdersAmtPaid.setColumnName("AMTPAID");
    colOrdersAmtPaid.setDataType(borland.jbcl.util.Variant.DOUBLE);
    colOrdersAmtPaid.setLocale(new java.util.Locale("en", "US", ""));
    colOrdersAmtPaid.setDefault("0");
    colOrdersAmtPaid.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersAmtPaid.addColumnChangeListener(new DataModule1_colOrdersAmtPaid_columnChangeAdapter(this));
    colOrdersShipName.setColumnName("SHIPNAME");
    colOrdersShipName.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersShipName.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersShipAddr1.setColumnName("SHIPADDR1");
    colOrdersShipAddr1.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersShipAddr1.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersShipAddr2.setColumnName("SHIPADDR2");
    colOrdersShipAddr2.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersShipAddr2.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersShipCity.setColumnName("SHIPCITY");
    colOrdersShipCity.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersShipCity.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersShipState.setColumnName("SHIPSTATE");
    colOrdersShipState.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersShipState.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersShipPostalCode.setColumnName("SHIPPOSTALCODE");
    colOrdersShipPostalCode.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersShipPostalCode.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersShipCountry.setColumnName("SHIPCOUNTRY");
    colOrdersShipCountry.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersShipCountry.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersBillAddr1.setColumnName("BILLADDR1");
    colOrdersBillAddr1.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersBillAddr1.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersBillAddr2.setColumnName("BILLADDR2");
    colOrdersBillAddr2.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersBillAddr2.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersBillCity.setColumnName("BILLCITY");
    colOrdersBillCity.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersBillCity.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersBillState.setColumnName("BILLSTATE");
    colOrdersBillState.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersBillState.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersBillPostalCode.setColumnName("BILLPOSTALCODE");
    colOrdersBillPostalCode.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersBillPostalCode.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersBillCountry.setColumnName("BILLCOUNTRY");
    colOrdersBillCountry.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersBillCountry.setVisible(borland.jbcl.util.TriState.NO);
    colOrdersCustomerName.setCalcType(borland.jbcl.dataset.CalcType.CALC);
    colOrdersCustomerName.setColumnName("CUSTOMERNAME");
    colOrdersCustomerName.setDataType(borland.jbcl.util.Variant.STRING);
    colOrdersCustomerName.setResolvable(false);
    colOrdersCustomerName.setVisible(borland.jbcl.util.TriState.NO);
    orderDataSet.setColumns(new Column[] {
      colOrdersID,
      colOrdersCustomerID,
      colOrdersOrderDate,
      colOrdersStatus,
      colOrdersShipDate,
      colOrdersOrderTrackNum,
      colOrdersCustomerPONum,
      colOrdersPayMethod,
      colOrdersCreditCardNum,
      colOrdersWirePaymentInstr,
      colOrdersShipMethod,
      colOrdersAmtPaid,
      colOrdersShipName,
      colOrdersShipAddr1,
      colOrdersShipAddr2,
      colOrdersShipCity,
      colOrdersShipState,
      colOrdersShipPostalCode,
      colOrdersShipCountry,
      colOrdersBillAddr1,
      colOrdersBillAddr2,
      colOrdersBillCity,
      colOrdersBillState,
      colOrdersBillPostalCode,
      colOrdersBillCountry,
      colOrdersCustomerName
    });
    // OrderItem
    colOrderItemQty.setCaption(res.getString("DM_Qty"));
    colOrderItemQty.setColumnName("QTY");
    colOrderItemQty.setDataType(borland.jbcl.util.Variant.INT);
    colOrderItemQty.setDefault("1");
    colOrderItemProductID.setCaption(res.getString("DM_Product_ID"));
    colOrderItemProductID.setColumnName("PRODUCTID");
    colOrderItemProductID.setDataType(borland.jbcl.util.Variant.INT);
    colOrderItemProductID.addColumnChangeListener(new DataModule1_colOrderItemProductID_columnChangeAdapter(this));

    // Create a pickList for the ProductID column of the OrderItem dataset
    colOrderItemProductID.setPickList(new borland.jbcl.dataset.PickListDescriptor(productDataSet, new String[] {"ID", "BASEPRICE"}, new String[] {"ID", "NAME", "BASEPRICE"}, new String[] {"PRODUCTID", "SALEPRICE"}, false));
    colOrderItemProductID.setItemEditor(edtOrderItemProductID);
    edtOrderItemProductID.setAllowSearch(false);
    edtOrderItemProductID.setDisplayOKCancel(true);
    colOrderItemProductName.setCaption(res.getString("DM_Name"));
    colOrderItemProductName.setColumnName("PRODUCTNAME");
    colOrderItemProductName.setDataType(borland.jbcl.util.Variant.STRING);
    colOrderItemProductName.setCalcType(borland.jbcl.dataset.CalcType.CALC);
    colOrderItemProductName.setEditable(false);
    colOrderItemProductName.setResolvable(false);

    colOrderItemSalePrice.setCaption(res.getString("DM_Sale_Price"));
    colOrderItemSalePrice.setCurrency(true);
    colOrderItemSalePrice.setColumnName("SALEPRICE");
    colOrderItemSalePrice.setDataType(borland.jbcl.util.Variant.DOUBLE);
    colOrderItemSalePrice.setLocale(new java.util.Locale("en", "US", ""));
    colOrderItemSalePrice.setDefault("0");
    colOrderItemExtendedPrice.setCalcType(borland.jbcl.dataset.CalcType.CALC);
    colOrderItemExtendedPrice.setCaption(res.getString("DM_Ext_Price"));
    colOrderItemExtendedPrice.setCurrency(true);
    colOrderItemExtendedPrice.setColumnName("EXTENDEDPRICE");
    colOrderItemExtendedPrice.setDataType(borland.jbcl.util.Variant.DOUBLE);
    colOrderItemExtendedPrice.setLocale(new java.util.Locale("en", "US", ""));
    colOrderItemExtendedPrice.setDefault("0");
    colOrderItemExtendedPrice.setReadOnly(true);
    colOrderItemExtendedPrice.setResolvable(false);
    colOrderItemSubtotal.setCalcType(borland.jbcl.dataset.CalcType.AGGREGATE);
    colOrderItemSubtotal.setCaption(res.getString("DM_SubTotal"));
    colOrderItemSubtotal.setCurrency(true);
    colOrderItemSubtotal.setColumnName("SUBTOTAL");
    colOrderItemSubtotal.setDataType(borland.jbcl.util.Variant.DOUBLE);
    colOrderItemSubtotal.setLocale(new java.util.Locale("en", "US", ""));
    colOrderItemSubtotal.setDefault("0");
    colOrderItemSubtotal.setReadOnly(true);
    colOrderItemSubtotal.setResolvable(false);
    // Set AggregationDescriptor for calculated column
    colOrderItemSubtotal.setAgg(new AggDescriptor( new String[] { "ORDERID" },
                                      "EXTENDEDPRICE",
                                      new SumAggOperator() ));
    colOrderItemOrderID.setCaption(res.getString("DM_Order_ID"));
    colOrderItemOrderID.setColumnName("ORDERID");
    colOrderItemOrderID.setDataType(borland.jbcl.util.Variant.INT);
    colOrderItemOrderID.setVisible(borland.jbcl.util.TriState.NO);
    colOrderItemTax.setCaption(res.getString("DM_Tax"));
    colOrderItemTax.setCurrency(true);
    colOrderItemTax.setCalcType(borland.jbcl.dataset.CalcType.AGGREGATE);
    colOrderItemTax.setColumnName("TAX");
    colOrderItemTax.setDataType(borland.jbcl.util.Variant.DOUBLE);
    colOrderItemTax.setLocale(new java.util.Locale("en", "US", ""));
    colOrderItemTax.setDefault("0");
    colOrderItemTax.setResolvable(false);
    // Set AggregationDescriptor for calculated column
    colOrderItemTax.setAgg(new AggDescriptor( new String[] { "ORDERID" },
                                      "EXTENDEDPRICE",
                                      null ));
    colOrderItemShipping.setCaption(res.getString("DM_Shipping"));
    colOrderItemShipping.setCurrency(true);
    colOrderItemShipping.setCalcType(borland.jbcl.dataset.CalcType.AGGREGATE);
    colOrderItemShipping.setColumnName("SHIPPING");
    colOrderItemShipping.setDataType(borland.jbcl.util.Variant.DOUBLE);
    colOrderItemShipping.setLocale(new java.util.Locale("en", "US", ""));
    colOrderItemShipping.setDefault("0");
    colOrderItemShipping.setResolvable(false);
    // Set AggregationDescriptor for calculated column
    colOrderItemShipping.setAgg(new AggDescriptor( new String[] { "ORDERID" },
                                      "EXTENDEDPRICE",
                                      null ));
    colOrderItemAmtDue.setCalcType(borland.jbcl.dataset.CalcType.AGGREGATE);
    colOrderItemAmtDue.setCaption(res.getString("DM_Amt_Due"));
    colOrderItemAmtDue.setCurrency(true);
    colOrderItemAmtDue.setColumnName("AMTDUE");
    colOrderItemAmtDue.setDataType(borland.jbcl.util.Variant.DOUBLE);
    colOrderItemAmtDue.setLocale(new java.util.Locale("en", "US", ""));
    colOrderItemAmtDue.setDefault("0");
    colOrderItemAmtDue.setResolvable(false);
    // Set AggregationDescriptor for calculated column
    colOrderItemAmtDue.setAgg(new AggDescriptor( new String[] { "ORDERID" },
                                       "EXTENDEDPRICE",
                                       null ));
    orderItemDataSet.setColumns(new Column[] {
      colOrderItemQty,
      colOrderItemProductID,
      colOrderItemProductName,
      colOrderItemSalePrice,
      colOrderItemExtendedPrice,
      colOrderItemSubtotal,
      colOrderItemOrderID,
      colOrderItemTax,
      colOrderItemShipping,
      colOrderItemAmtDue
    });
    // Customer
    colCustomerID.setCaption(res.getString("DM_CustomerDS_ID"));
    colCustomerID.setColumnName("ID");
    colCustomerID.setDataType(borland.jbcl.util.Variant.INT);
    colCustomerFirstName.setCaption(res.getString("DM_CustomerDS_FIRSTNAME"));
    colCustomerFirstName.setColumnName("FIRSTNAME");
    colCustomerFirstName.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerMI.setCaption(res.getString("DM_CustomerDS_MI"));
    colCustomerMI.setColumnName("MI");
    colCustomerMI.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerLastName.setCaption(res.getString("DM_CustomerDS_LASTNAME"));
    colCustomerLastName.setColumnName("LASTNAME");
    colCustomerLastName.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerPhone.setCaption(res.getString("DM_CustomerDS_PHONE"));
    colCustomerPhone.setColumnName("PHONE");
    colCustomerPhone.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerFax.setColumnName("FAX");
    colCustomerFax.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerFax.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerEmail.setColumnName("EMAIL");
    colCustomerEmail.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerEmail.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerAddr1.setColumnName("ADDR1");
    colCustomerAddr1.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerAddr1.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerAddr2.setColumnName("ADDR2");
    colCustomerAddr2.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerAddr2.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerCity.setColumnName("CITY");
    colCustomerCity.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerCity.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerState.setColumnName("STATE");
    colCustomerState.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerState.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerPostalCode.setColumnName("POSTALCODE");
    colCustomerPostalCode.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerPostalCode.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerCountry.setColumnName("COUNTRY");
    colCustomerCountry.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerCountry.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerShipName.setColumnName("SHIPNAME");
    colCustomerShipName.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerShipName.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerShipAddr1.setColumnName("SHIPADDR1");
    colCustomerShipAddr1.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerShipAddr1.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerShipAddr2.setColumnName("SHIPADDR2");
    colCustomerShipAddr2.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerShipAddr2.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerShipCity.setColumnName("SHIPCITY");
    colCustomerShipCity.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerShipCity.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerShipState.setColumnName("SHIPSTATE");
    colCustomerShipState.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerShipState.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerShipPostalCode.setColumnName("SHIPPOSTALCODE");
    colCustomerShipPostalCode.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerShipPostalCode.setVisible(borland.jbcl.util.TriState.NO);
    colCustomerShipCountry.setColumnName("SHIPCOUNTRY");
    colCustomerShipCountry.setDataType(borland.jbcl.util.Variant.STRING);
    colCustomerShipCountry.setVisible(borland.jbcl.util.TriState.NO);
    customerDataSet.setColumns(new Column[] {
      colCustomerID,
      colCustomerFirstName,
      colCustomerMI,
      colCustomerLastName,
      colCustomerPhone,
      colCustomerFax,
      colCustomerEmail,
      colCustomerAddr1,
      colCustomerAddr2,
      colCustomerCity,
      colCustomerState,
      colCustomerPostalCode,
      colCustomerCountry,
      colCustomerShipName,
      colCustomerShipAddr1,
      colCustomerShipAddr2,
      colCustomerShipCity,
      colCustomerShipState,
      colCustomerShipPostalCode,
      colCustomerShipCountry
    });
    // Customer Order
    colCustOrderID.setColumnName("ID");
    colCustOrderID.setDataType(borland.jbcl.util.Variant.INT);
    colCustOrderID.setVisible(borland.jbcl.util.TriState.NO);
    colCustOrderCustomerID.setColumnName("CUSTOMERID");
    colCustOrderCustomerID.setDataType(borland.jbcl.util.Variant.INT);
    colCustOrderCustomerID.setVisible(borland.jbcl.util.TriState.NO);
    colCustOrderOrderDate.setCaption(res.getString("DM_CustOrderDS_ORDERDATE"));
    colCustOrderOrderDate.setColumnName("ORDERDATE");
    colCustOrderOrderDate.setDisplayMask(res.getString("DM_DateDisplayMask"));
    colCustOrderOrderDate.setDataType(borland.jbcl.util.Variant.TIMESTAMP);
    colCustOrderStatus.setCaption(res.getString("DM_CustOrderDS_STATUS"));
    colCustOrderStatus.setColumnName("STATUS");
    colCustOrderStatus.setDataType(borland.jbcl.util.Variant.STRING);
    colCustOrderStatus.addColumnChangeListener(new DataModule1_colOrdersStatus_columnChangeAdapter(this));
    colCustOrderShipDate.setCaption(res.getString("DM_CustOrderDS_SHIPDATE"));
    colCustOrderShipDate.setColumnName("SHIPDATE");
    colCustOrderShipDate.setDisplayMask(res.getString("DM_DateDisplayMask"));
    colCustOrderShipDate.setDataType(borland.jbcl.util.Variant.TIMESTAMP);
    custOrderDataSet.setColumns(new Column[] {
      colCustOrderID,
      colCustOrderCustomerID,
      colCustOrderOrderDate,
      colCustOrderStatus,
      colCustOrderShipDate
    });
    // Product
    colProductID.setCaption(res.getString("DM_ID"));
    colProductID.setColumnName("ID");
    colProductID.setDataType(borland.jbcl.util.Variant.INT);
    colProductIsActive.setCaption(res.getString("DM_Is_Active"));
    colProductIsActive.setColumnName("ISACTIVE");
    colProductIsActive.setDataType(borland.jbcl.util.Variant.SHORT);
    colProductIsActive.setDefault("1");
    colProductName.setCaption(res.getString("DM_Name"));
    colProductName.setColumnName("NAME");
    colProductName.setDataType(borland.jbcl.util.Variant.STRING);
    colProductCategory.setCaption(res.getString("DM_Category"));
    colProductCategory.setColumnName("CATEGORY");
    colProductCategory.setDataType(borland.jbcl.util.Variant.STRING);
    colProductCategory.setDefault(res.getString("DM_Active_Wear"));
    colProductBasePrice.setCaption(res.getString("DM_Base_Price"));
    colProductBasePrice.setDisplayMask("#0.00");
    colProductBasePrice.setLocale(new java.util.Locale("en", "US", ""));
    colProductBasePrice.setColumnName("BASEPRICE");
    colProductBasePrice.setDataType(borland.jbcl.util.Variant.DOUBLE);
    colProductBasePrice.setDefault("0");
    colProductBasePrice.setVisible(borland.jbcl.util.TriState.NO);
    colProductDiscountPct.setCaption(res.getString("DM_Discount_%"));
    colProductDiscountPct.setColumnName("DISCOUNTPCT");
    colProductDiscountPct.setDisplayMask("#0.00");
    colProductDiscountPct.setMax("100");
    colProductDiscountPct.setDataType(borland.jbcl.util.Variant.DOUBLE);
    colProductDiscountPct.setDefault("0");
    colProductDiscountPct.setMin("0");
    colProductDiscountPct.setVisible(borland.jbcl.util.TriState.NO);
    colProductStockQty.setCaption(res.getString("DM_Stock_Qty"));
    colProductStockQty.setColumnName("STOCKQTY");
    colProductStockQty.setDataType(borland.jbcl.util.Variant.INT);
    colProductStockQty.setDefault("0");
    colProductMinReorderQty.setCaption(res.getString("DM_Min_ReOrder_Qty"));
    colProductMinReorderQty.setColumnName("MINREORDERQTY");
    colProductMinReorderQty.setDataType(borland.jbcl.util.Variant.INT);
    colProductMinReorderQty.setDefault("0");
    colProductMinReorderQty.setVisible(borland.jbcl.util.TriState.NO);
    productDataSet.setColumns(new Column[] {
      colProductID,
      colProductIsActive,
      colProductName,
      colProductCategory,
      colProductBasePrice,
      colProductDiscountPct,
      colProductStockQty,
      colProductMinReorderQty
    });
    categoryColumn.setCaption(res.getString("DM_CategoryDS_CATEGORY"));
    categoryColumn.setColumnName("CATEGORY");
    categoryColumn.setDataType(borland.jbcl.util.Variant.STRING);
    categoryDataSet.setColumns(new Column[] {categoryColumn});
  }



  // Getter methods for Data Access controls

  /**
   * Class method to access the singleton instance of the class.
   */
  static public DataModule1 getDataModule() {
    if (myDM == null)
      myDM = new DataModule1();
    return myDM;
  }

  /**
   * Method to access the Product DataSet.
   */
  public borland.sql.dataset.QueryDataSet getProductDataSet() {
    return productDataSet;
  }

  /**
   * Method to access the Category DataSet.
   */
  public borland.sql.dataset.QueryDataSet getCategoryDataSet() {
    return categoryDataSet;
  }

  /**
   * Method to access the Customer DataSet.
   */
  public borland.sql.dataset.QueryDataSet getCustomerDataSet() {
    return customerDataSet;
  }

  /**
   * Method to access the Order DataSet that is linked to the
   * Customer DataSet by a MasterLink.
   */
  public borland.sql.dataset.QueryDataSet getCustOrderDataSet() {
    return custOrderDataSet;
  }

  /**
   * Method to access the Order DataSet.
   */
  public borland.sql.dataset.QueryDataSet getOrderDataSet() {
    return orderDataSet;
  }

  /**
   * Method to access the OrderItem DataSet. This is linked to the
   * Order DataSet by a MasterLink.
   */
  public borland.sql.dataset.QueryDataSet getOrderItemDataSet() {
    return orderItemDataSet;
  }

  /**
   * Method to access the PayMethod DataSet.
   */
  public borland.sql.dataset.QueryDataSet getPayMethodDataSet() {
    return payMethodDataSet;
  }

  /**
   * Method to access the ShipMethod DataSet.
   */
  public borland.sql.dataset.QueryDataSet getShipMethodDataSet() {
    return shipMethodDataSet;
  }

  /**
   * Method to access the Status DataSet.
   */
  public borland.sql.dataset.QueryDataSet getStatusDataSet() {
    return statusDataSet;
  }

  /**
   * Method to access the Account DataSet.
   */
  public borland.sql.dataset.QueryDataSet getAccountDataSet() {
    return accountDataSet;
  }

  /**
   * This method is used to open a connection to the database
   * explicitly.
   */
  public void dbConnect() throws DataSetException {
    database1.openConnection();
  }


  // Category DataSet EditEvents

  /**
   * Before adding a category row, do row-level validation.
   */
  void categoryDataSet_adding(DataSet dataSet, ReadWriteRow readWriteRow) throws Exception{
    validateCategoryRow();
  }

  /**
   * Before updating a category row, do row-level validation.
   */
  void categoryDataSet_updating(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow) throws Exception{
    validateCategoryRow();
  }

  /**
   * Before deleting a Category record, prompt the user to confirm delete.
   */
  void categoryDataSet_deleting(DataSet dataSet) throws Exception{
    confirmDelete(res.getString("DM_Delete_this_category?"));
  }

  /**
   * When a new record is added, resolve new record to the database.
   */
  void categoryDataSet_added(DataSet dataSet) throws DataSetException{
    dataSet.saveChanges();
  }

  /**
   * When a record is updated, resolve updated record to the database.
   */
  void categoryDataSet_updated(DataSet dataSet) throws DataSetException{
    dataSet.saveChanges();
  }

  /**
   * When a record is deleted, resolve to the database.
   */
  void categoryDataSet_deleted(DataSet dataSet) throws DataSetException{
    dataSet.saveChanges();
  }

  /**
   * When canceling an edit or insert, clear the messages from
   * all status bar controls
   */
  void categoryDataSet_canceling(DataSet dataSet) throws Exception{
    dataSet.clearStatus();
  }

  /**
   * Show an error dialog if error occurs during editing.
   */
  void categoryDataSet_editError(DataSet dataSet, Column column, Variant variant, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }

  /**
   * Show an error dialog if error occurs during adding.
   */
  void categoryDataSet_addError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }

  /**
   * Show an error dialog if error occurs during updating.
   */
  void categoryDataSet_updateError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }



  // Product DataSet EditEvents

  /**
   * Before adding a product row, do row-level validation.
   */
  void productDataSet_adding(DataSet dataSet, ReadWriteRow readWriteRow) throws Exception{
    validateProductRow();
  }

  /**
   * Before updating a product row, do row-level validation.
   */
  void productDataSet_updating(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow) throws Exception{
    validateProductRow();
  }

  /**
   * Before deleting a Product record, prompt the user to confirm delete.
   */
  void productDataSet_deleting(DataSet dataSet) throws Exception{
    confirmDelete(res.getString("DM_Delete_this_product?"));
  }

  /**
   * When a new record is added, resolve new record to the database.
   */
  void productDataSet_added(DataSet dataSet) throws DataSetException{
    dataSet.saveChanges();
  }

  /**
   * When a record is updated, resolve updated record to the database.
   */
  void productDataSet_updated(DataSet dataSet) throws DataSetException{
    dataSet.saveChanges();
  }

  /**
   * When a record is deleted, resolve to the database.
   */
  void productDataSet_deleted(DataSet dataSet) throws DataSetException{
    dataSet.saveChanges();
  }

  /**
   * When canceling an edit or insert, clear the messages from
   * all status bar controls
   */
  void productDataSet_canceling(DataSet dataSet) throws Exception{
    dataSet.clearStatus();
  }

  /**
   * Show an error dialog if error occurs during editing.
   */
  void productDataSet_editError(DataSet dataSet, Column column, Variant variant, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }

  /**
   * Show an error dialog if error occurs during adding.
   */
  void productDataSet_addError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }

  /**
   * Show an error dialog if error occurs during updating.
   */
  void productDataSet_updateError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }



  // Customer DataSet EditEvents

  /**
   * Before adding a new customer record, get the Order ID from the
   * Server. We want the ID upfront for the Order Item records.
   * Also, copy the customer info into the ShipTo info for the
   * customer if the ShipName column is NULL.
   */
  void customerDataSet_adding(DataSet dataSet, ReadWriteRow readWriteRow) throws Exception{
     validateCustomerRow();

    // Call the stored procedure to get a new Customer ID
    int customerID = getNextCustomerID();
    readWriteRow.setInt("ID", customerID);

    // Copy customer info into shipto info columns
    if (readWriteRow.isNull("SHIPNAME")) {
      String custName = formatCustomerName( readWriteRow.getString("FIRSTNAME"),
                                            readWriteRow.getString("MI"),
                                            readWriteRow.getString("LASTNAME") );
      readWriteRow.setString("SHIPNAME",    custName);
      readWriteRow.setString("SHIPADDR1",   readWriteRow.getString("ADDR1"));
      readWriteRow.setString("SHIPADDR2",   readWriteRow.getString("ADDR2"));
      readWriteRow.setString("SHIPCITY",    readWriteRow.getString("CITY"));
      readWriteRow.setString("SHIPSTATE",   readWriteRow.getString("STATE"));
      readWriteRow.setString("SHIPPOSTALCODE", readWriteRow.getString("POSTALCODE"));
      readWriteRow.setString("SHIPCOUNTRY", readWriteRow.getString("COUNTRY"));
    }
  }

  /**
   * Method to get the next customer ID.
   * This calls a stored procedure on the database server spNextCustomerID
   * that returns a single row with the next Customer ID generated by the server.
   */
  private int getNextCustomerID() throws Exception {
    int result = 0;
    try {
      nextCustomerIDDataSet.open();
      nextCustomerIDDataSet.refresh();
      result = nextCustomerIDDataSet.getInt("NEXTID");
      nextCustomerIDDataSet.close();
    }
    catch (Exception ex) {
      ex.printStackTrace();
      throw ex;
    }
    return result;
  }

  /**
   * Before updating a customer row, do row-level validation.
   */
  void customerDataSet_updating(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow) throws Exception{
    validateCustomerRow();
  }

  /**
   * Before deleting a Customer record, prompt the user to confirm delete.
   */
  void customerDataSet_deleting(DataSet dataSet) throws Exception{
    confirmDelete(res.getString("DM_Delete_this_customer?"));
  }

  /**
   * When a new record is added, resolve new record to the database.
   */
  void customerDataSet_added(DataSet dataSet) throws DataSetException{
    dataSet.saveChanges();
  }

  /**
   * When a record is updated, resolve updated record to the database.
   */
  void customerDataSet_updated(DataSet dataSet) throws DataSetException{
    dataSet.saveChanges();
  }

  /**
   * When a record is deleted, resolve to the database.
   */
  void customerDataSet_deleted(DataSet dataSet) throws DataSetException{
    dataSet.saveChanges();
  }

  /**
   * When canceling an edit or insert, clear the messages from
   * all status bar controls
   */
  void customerDataSet_canceling(DataSet dataSet) throws Exception{
    dataSet.clearStatus();
  }

  /**
   * Show an error dialog if error occurs during editing.
   */
  void customerDataSet_editError(DataSet dataSet, Column column, Variant variant, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }

  /**
   * Show an error dialog if error occurs during adding.
   */
  void customerDataSet_addError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }

  /**
   * Show an error dialog if error occurs during updating.
   */
  void customerDataSet_updateError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }




  // Order DataSet EditEvents

  /**
   * When adding a new order record, get the Order ID from the
   * Server. We want the ID upfront for the Order Item records.
   */
  void orderDataSet_adding(DataSet dataSet, ReadWriteRow readWriteRow) throws Exception{
    validateOrderRow();

    // call the stored procedure to get a new Order ID
    int orderID = getNextOrderID();
    readWriteRow.setInt("ID", orderID);
  }

  /**
   * Method to get the next order ID.
   * This calls a stored procedure on the database server spNextOrderID
   * that returns a single row with the next Order ID generated by the server.
   */
  private int getNextOrderID() throws Exception {
    int result = 0;
    try {
      nextOrderIDDataSet.open();
      nextOrderIDDataSet.refresh();
      result = nextOrderIDDataSet.getInt("NEXTID");
      nextOrderIDDataSet.close();
    }
    catch (Exception ex) {
      ex.printStackTrace();
      throw ex;
    }
    return result;
  }

  /**
   * Before updating an order row, do row-level validation.
   */
  void orderDataSet_updating(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow) throws Exception{
    validateOrderRow();
  }

  /**
   * When canceling an edit or insert, clear the messages from
   * all status bar controls
   */
  void orderDataSet_canceling(DataSet dataSet) throws Exception{
    dataSet.clearStatus();
  }

  /**
   * Before deleting an Order record, prompt the user to confirm delete.
   */
  void orderDataSet_deleting(DataSet dataSet) throws Exception{
    confirmDelete(res.getString("DM_Delete_this_order?"));
  }

  /**
   * Show an error dialog if error occurs during editing.
   */
  void orderDataSet_editError(DataSet dataSet, Column column, Variant variant, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }

  /**
   * Show an error dialog if error occurs during adding.
   */
  void orderDataSet_addError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }

  /**
   * Show an error dialog if error occurs during updating.
   */
  void orderDataSet_updateError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }




  // OrderItem DataSet EditEvents

  /**
   * Before inserting a new OrderItem record, post the Order dataset
   * if a new record is being inserted.
   */
  void orderItemDataSet_inserting(DataSet dataSet) throws Exception{
    // Do not allow inserts if the corresponding Order record
    // STATUS value is Fullfilled.
    String status = orderDataSet.getString("STATUS");
    if (status.compareTo(res.getString("DM_Fulfilled")) == 0) {
      // Show the message dialog here because there is
      // no error event handler for inserting event errors.
      CliffhangerApplication.messageDlg(
        res.getString("DM_Error"),
        res.getString("DM_Cannot_modify_order"),
        Message.OK);
      throw new VetoException(res.getString("DM_Cannot_modify_order"));
    }

    // Post order dataset first
    if (orderDataSet.isEditingNewRow()) {
      orderDataSet.post();
    }
  }

  /**
   * The adding event occurs after the inserted record is posted.
   * Before an orderItem is added, check the quantity on hand in the
   * Product dataset and subtract the quantity for this order item.
   */
  void orderItemDataSet_adding(DataSet dataSet, ReadWriteRow readWriteRow) throws Exception{
    validateOrderItemRow();

    int productID  = readWriteRow.getInt("PRODUCTID");
    int orderedQty = readWriteRow.getInt("QTY");

    // Update the product StockQty by the amount ordered
    subtractProductStockQty(productID, orderedQty);
  }

  /**
   * The updating event occurs after the modified record is posted.
   * When an orderItem is being updated, do the following:
   * <UL>
   * <LI>1. Determine if the productID has changed
   *    <P>
   *    - subtract qty from new product
   *    <P>
   *    - replace qty of old product
   * <LI>2. Determine if the quantity has changed
   *    <P>
   *    - replace or subtract from product
   * </UL>
   */
  void orderItemDataSet_updating(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow) throws Exception{
    validateOrderItemRow();

    int oldProductID  = readRow.getInt("PRODUCTID");
    int oldOrderedQty = readRow.getInt("QTY");
    int newProductID  = readWriteRow.getInt("PRODUCTID");
    int newOrderedQty = readWriteRow.getInt("QTY");

    if (oldProductID != newProductID) {
      subtractProductStockQty(newProductID, newOrderedQty);
      subtractProductStockQty(oldProductID, -oldOrderedQty);
    }
    else {
      if (newOrderedQty != oldOrderedQty)
        subtractProductStockQty(newProductID, newOrderedQty - oldOrderedQty);
    }
  }

  /**
   * The deleting event occurs before a record is deleted.
   * Confirm delete with the user, and update stock quantity.
   */
  void orderItemDataSet_deleting(DataSet dataSet) throws Exception{
    // Prompt the user to confirm delete.
    confirmDelete(res.getString("DM_Delete_this_order"));

    // Do not allow edits if the corresponding Order record
    // STATUS value is Fulfilled.
    String status = orderDataSet.getString("STATUS");
    if (status.compareTo(res.getString("DM_Fulfilled")) == 0) {
      // Show the message dialog here because there is
      // no error event handler for modifying event errors.
      CliffhangerApplication.messageDlg(
        res.getString("DM_Error"),
        res.getString("DM_Cannot_modify_order"),
        Message.OK);
      throw new VetoException(res.getString("DM_Cannot_modify_order"));
    } else {
      // When an orderItem is being deleted, replace the qty of the product.
      int productID  = dataSet.getInt("PRODUCTID");
      int orderedQty = dataSet.getInt("QTY");
      subtractProductStockQty(productID, -orderedQty);
    }
  }

  /**
   * Method to prompt the user to confirm before deleting a record.
   * This raises an exception if the user cancels.
   * Call this method from the deleting event handler.
   */
  private void confirmDelete(String message) throws Exception {
    // Prompt the user to confirm delete.
    int answer = CliffhangerApplication.messageDlg(
                   res.getString("DM_Confirm"),
                   message,
                   Message.YES_NO);

    if (answer == Message.NO) {
      throw new VetoException(res.getString("DM_Delete_canceled"));
    }
  }

  /**
   * Event handler for modifying event. Make sure that the orderDataSet's
   * Stutus column is not "Fulfilled" before allowing the orderItemDataSet
   * to be modified.
   */
  void orderItemDataSet_modifying(DataSet dataSet) throws Exception{
    // Do not allow edits if the corresponding Order record
    // STATUS value is Fulfilled.
    String status = orderDataSet.getString("STATUS");
    if (status.compareTo(res.getString("DM_Fulfilled")) == 0) {
      // Show the message dialog here because there is
      // no error event handler for modifying event errors.
      CliffhangerApplication.messageDlg(
        res.getString("DM_Error"),
        res.getString("DM_Cannot_modify_order"),
        Message.OK);
      throw new VetoException(res.getString("DM_Cannot_modify_order"));
    }
  }

  /**
   * When canceling an edit or insert, clear the messages from
   * all status bar controls
   */
  void orderItemDataSet_canceling(DataSet dataSet) throws Exception{
    dataSet.clearStatus();
  }

  /**
   * Event handler to calculate the values for all the calculated columns
   * for the OrderItem dataset.
   */
  void orderItemDataSet_calcFields(ReadRow readRow, DataRow dataRow, boolean boolean1) throws DataSetException{
    // Calculate the Extended Qty for OrderItems
    double extPrice = readRow.getInt("QTY") * readRow.getDouble("SALEPRICE");
    dataRow.setDouble("EXTENDEDPRICE", extPrice);

    // Lookup the product Name for the OrderItem
    String productName = lookupProductName(readRow.getInt("PRODUCTID"));
    dataRow.setString("PRODUCTNAME", productName);
  }

  /**
   * Method to lookup the Product Name given the ProductID.
   */
  private String lookupProductName(int productID) {
    String productName = new String("");
    try {
      // Define a DataRow to hold the productID to look for
      // the Product Dataset, and another to hold the row
      // of product data that we find.
      productDataSet.open();
      DataRow lookupRow = new DataRow(productDataSet, "ID");
      DataRow productRow = new DataRow(productDataSet, new String[] {"NAME"} );

      lookupRow.setInt("ID", productID);

      if (productDataSet.lookup(lookupRow, productRow, Locate.FIRST)) {
        productName = productRow.getString("NAME");
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    };
    return productName;
  }

  /**
   * Event handler to calculate the aggregate columns for the
   * orderItem DataSet when an row is added.
   */
  void orderItemDataSet_calcAggAdd(ReadRow readRow, ReadWriteRow readWriteRow) throws DataSetException{
    calcAggregate(readRow, readWriteRow);
  }

  /**
   * Event handler to calculate the aggregate columns for the
   * orderItem DataSet when an row is deleted.
   */
  void orderItemDataSet_calcAggDelete(ReadRow readRow, ReadWriteRow readWriteRow) throws DataSetException{
    calcAggregate(readRow, readWriteRow);
  }

  /**
   * Method to calculate the aggregate columns
   * called by calcAggAdd and calcAggDelete.
   */
  void calcAggregate(ReadRow readRow, ReadWriteRow readWriteRow) throws DataSetException {
    double subTotal = readRow.getDouble("SUBTOTAL");
    // Calculate the Tax on the SubTotal
    double tax = subTotal * TAXPERCENT;
    readWriteRow.setDouble("TAX", tax);
    // Calculate the Shipping on the SubTotal
    double shipping = subTotal * SHIPPERCENT;
    readWriteRow.setDouble("SHIPPING", shipping);
    // Calculate the AmtDue                                      
    double amtPaid = orderDataSet.getDouble("AMTPAID");
    readWriteRow.setDouble("AMTDUE", ((subTotal + tax + shipping) - amtPaid));
  }

  /**
   * Event handler to calculate the values for all the calculated columns
   * for the Orders dataset.
   */
  void orderDataSet_calcFields(ReadRow readRow, DataRow dataRow, boolean boolean1) throws DataSetException{
    // Lookup the CustomerName for the current Order
    String custName = lookupCustomerName(readRow.getInt("CUSTOMERID"));
    dataRow.setString("CUSTOMERNAME", custName);
  }

  /**
   * Method to lookup the CustomerName given the customerID
   * This method also formats the name, taking care of the
   * middle initial.
   */
  private String lookupCustomerName(int customerID) {
    String customerName = new String("");
    try {
      // Define a DataRow to hold the customerID to look for
      // the Customer Dataset, and another to hold the row
      // of customer data that we find.
      customerDataSet.open();
      DataRow lookupRow = new DataRow(customerDataSet, "ID");
      DataRow customerRow = new DataRow(customerDataSet, new String[] {"FIRSTNAME", "MI", "LASTNAME"} );

      lookupRow.setInt("ID", customerID);

      if (customerDataSet.lookup(lookupRow, customerRow, Locate.FIRST)) {
        String firstName = customerRow.getString("FIRSTNAME");
        String mi        = customerRow.getString("MI");
        String lastName  = customerRow.getString("LASTNAME");

        // Format the customer name
        customerName = formatCustomerName(firstName, mi, lastName);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    };
    return customerName;
  }

  /**
   * Method to format a customer name given the first name, middle initial,
   * and last name.
   */
  private String formatCustomerName(String firstName, String mi, String lastName) {
    // Format the customer name
    if (mi.compareTo("") == 0)
      return (firstName + " " + lastName);
    else
      return (firstName + " " + mi + ". " + lastName);
  }

  /**
   * Event handler when the user changes the ProductID, update the SalePrice
   * accordingly by looking up the PRICE for the Product.
   */
  void colOrderItemProductID_changed(DataSet dataSet, Column column, Variant variant) throws DataSetException{
    double salePrice = lookupProductSalePrice(dataSet.getInt("PRODUCTID"));
    try {
      dataSet.startEdit(column);
      dataSet.setDouble("SALEPRICE", salePrice);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  /**
   * Method to lookup the Product SalePrice given the ProductID.
   */
  private double lookupProductSalePrice(int productID) {
    double productSalePrice = 0.0;
    try {
      // Define a DataRow to hold the productID to look for
      // the Product Dataset, and another to hold the row
      // of product data that we find.
      productDataSet.open();
      DataRow lookupRow = new DataRow(productDataSet, "ID");
      DataRow productRow = new DataRow(productDataSet, new String[] {"BASEPRICE", "DISCOUNTPCT"} );

      lookupRow.setInt("ID", productID);

      if (productDataSet.lookup(lookupRow, productRow, Locate.FIRST)) {
        double basePrice   = productRow.getDouble("BASEPRICE");
        double discountPct = productRow.getDouble("DISCOUNTPCT");
        productSalePrice   = basePrice - (basePrice * (discountPct / 100));
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    };
    return productSalePrice;
  }


  /**
   * Show an error dialog if error occurs during editing.
   */
  void orderItemDataSet_editError(DataSet dataSet, Column column, Variant variant, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }

  /**
   * Show an error dialog if error occurs during adding.
   */
  void orderItemDataSet_addError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }

  /**
   * Show an error dialog if error occurs during updating.
   */
  void orderItemDataSet_updateError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    CliffhangerApplication.messageDlg(
      res.getString("DM_Error"),
      dataSetException.getMessage(),
      Message.OK);
  }



  /**
   * Event handler when the CustomerID has changed get the Customer info
   * for the new customer ID.
   */
  void colOrdersCustomerID_changed(DataSet dataSet, Column column, Variant variant) throws DataSetException{
    try {
      // Define a DataRow to hold the productID to look for
      // the Product Dataset, and another to hold the row
      // of product data that we find.
      customerDataSet.open();
      DataRow lookupRow = new DataRow(customerDataSet, "ID");
      DataRow customerRow = new DataRow(
        customerDataSet,
        new String[] {
          "ADDR1",
          "ADDR2",
          "CITY",
          "STATE",
          "POSTALCODE",
          "COUNTRY",
          "SHIPNAME",
          "SHIPADDR1",
          "SHIPADDR2",
          "SHIPCITY",
          "SHIPSTATE",
          "SHIPPOSTALCODE",
          "SHIPCOUNTRY"
        }
      );
      lookupRow.setInt("ID", dataSet.getInt("CUSTOMERID"));

      if (customerDataSet.lookup(lookupRow, customerRow, Locate.FIRST)) {
        dataSet.startEdit(column);
        dataSet.setString("BILLADDR1",      customerRow.getString("ADDR1"));
        dataSet.setString("BILLADDR2",      customerRow.getString("ADDR2"));
        dataSet.setString("BILLCITY",       customerRow.getString("CITY"));
        dataSet.setString("BILLSTATE",      customerRow.getString("STATE"));
        dataSet.setString("BILLPOSTALCODE", customerRow.getString("POSTALCODE"));
        dataSet.setString("BILLCOUNTRY",    customerRow.getString("COUNTRY"));
        dataSet.setString("SHIPNAME",       customerRow.getString("SHIPNAME"));
        dataSet.setString("SHIPADDR1",      customerRow.getString("SHIPADDR1"));
        dataSet.setString("SHIPADDR2",      customerRow.getString("SHIPADDR2"));
        dataSet.setString("SHIPCITY",       customerRow.getString("SHIPCITY"));
        dataSet.setString("SHIPSTATE",      customerRow.getString("SHIPSTATE"));
        dataSet.setString("SHIPPOSTALCODE", customerRow.getString("SHIPPOSTALCODE"));
        dataSet.setString("SHIPCOUNTRY",    customerRow.getString("SHIPCOUNTRY"));
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    };
  }

  /**
   * NOTE:
   * The issue of applying changes to the quantity of a product in the
   * Product table can be approached in two ways:
   * 1. Apply quantity changes to our client Product dataset and resolve
   *    the changes together when we resolve Orders and OrderItem datasets.
   * 2. Create a trigger on the database server for a BeforeInsert on the
   *    OrderItem table which could in turn call a stored procedure to
   *    make the necessary quantity changes to the Product table.
   * We have chosen to use method #1 to show case how you could do so on the
   * client, although it could be more realistic to use a stored procedure.
   */


  /**
   * This method updates the StockQty column of the Product dataset
   * checking that the resulting StockQty value does not go below 0.
   */
  private void subtractProductStockQty(int productID, int orderedQty) throws Exception {
    // Define a DataRow to hold the productID to locate in
    // the Product Dataset.
    productDataSet.open();
    DataRow locateRow = new DataRow(productDataSet, "ID");
    locateRow.setInt("ID", productID);

    if (productDataSet.locate(locateRow, Locate.FIRST)) {
      int stockQty = productDataSet.getInt("STOCKQTY");
      // make sure that (stockQty - orderedQty) >= 0
      int remainingQty = stockQty - orderedQty;
      if (remainingQty < 0) {
        throw new DataSetException(DataSetException.FIELD_POST_ERROR,
                                   res.getString("DM_Insufficient_stock"));
      }
      else {
        // make the changes to the product dataset
        productDataSet.editRow();
        productDataSet.setInt("STOCKQTY", remainingQty);
        productDataSet.post();
      } //else
    } //if
  }

  /**
   * This method is called when an exception is thrown during resolution
   * of modifications to a row in the Product DataSet.
   * Here we will try to refetch the row and retry the resolution.
   */
  void resProduct_updateError(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow, ReadWriteRow readWriteRow1, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    if (dataSetException.getErrorCode() == DataSetException.NO_ROWS_AFFECTED) {
      // refetch the row and make the necessary changes.
      dataSet.refetchRow(readWriteRow1);
      int updStockQty = readWriteRow1.getInt("STOCKQTY");

      // Determine the change in the row
      int oldStockQty = readRow.getInt("STOCKQTY");
      int newStockQty = readWriteRow.getInt("STOCKQTY");
      int qtyDifference = newStockQty - oldStockQty;

      // We will only attempt to make changes to the StockQty column here
      if (oldStockQty != updStockQty) {
        int updRemainingQty = updStockQty + qtyDifference;
        if (updRemainingQty >= 0) {
          readWriteRow.setInt("STOCKQTY", updRemainingQty);
          errorResponse.retry();
        }
        else
          errorResponse.abort();
      }
    }
  }

  /**
   * Do some row validation before adding new Order record
   * or before updating Order record.
   */
  private void validateOrderRow() throws Exception {
    // In validating an Order record check the following:
    // - must have valid CustomerId
    // - must have valid OrderDate
    // - must have valid Status
    if ( orderDataSet.isNull("CUSTOMERID") || (orderDataSet.getInt("CUSTOMERID") <= 0) )
      throw new VetoException(res.getString("DM_Invalid_customer"));
    if ( orderDataSet.isNull("ORDERDATE") )
      throw new VetoException(res.getString("DM_Invalid_order_date"));
    if ( orderDataSet.isNull("STATUS") || (orderDataSet.getString("STATUS").trim().compareTo("") == 0))
      throw new VetoException(res.getString("DM_Invalid_status"));
    if ( orderDataSet.isNull("PAYMETHOD") || (orderDataSet.getString("PAYMETHOD").trim().compareTo("") == 0))
      throw new VetoException(res.getString("DM_Invalid_paymethod"));
  }

  /**
   * Do some row validation before adding new OrderItem record
   * or before updating OrderItem record.
   */
  private void validateOrderItemRow() throws Exception {
    // validate only if posting from editing or inserting
    //if (!(orderItemDataSet.isEditing() || orderItemDataSet.isEditingNewRow())) return;

    // In validating an OrderItem record check the following:
    // - must have valid ProductId
    // - must have valid Qty
    if ( orderItemDataSet.isNull("PRODUCTID") || (orderItemDataSet.getInt("PRODUCTID") <= 0) )
      throw new VetoException(res.getString("DM_Invalid_product"));
    if ( orderItemDataSet.isNull("QTY") || (orderItemDataSet.getInt("QTY") <= 0) )
      throw new VetoException(res.getString("DM_Invalid_product_quantity"));
  }

  /**
   * Do some row validation before adding new product record
   * or before updating product record.
   */
  private void validateProductRow() throws Exception{
    // In validating an product record check the following:
    // - must have valid Name
    // - must have valid Category
    // - must have valid BasePrice
    // - must have valid DiscountPct, but min and max contraints take care of that
    // - must have valid StockQty
    // - must have valid MinReorderQty
    if ( productDataSet.isNull("NAME") || (productDataSet.getString("NAME").trim().compareTo("") == 0) )
      throw new VetoException(res.getString("DM_Invalid_product_name"));
    if ( productDataSet.isNull("CATEGORY") || (productDataSet.getString("CATEGORY").trim().compareTo("") == 0) )
      throw new VetoException(res.getString("DM_Invalid_product_category"));
    if ( productDataSet.isNull("BASEPRICE") || (productDataSet.getDouble("BASEPRICE") < 0.0) )
      throw new VetoException(res.getString("DM_Invalid_base_price"));
    if ( productDataSet.isNull("STOCKQTY") || (productDataSet.getInt("STOCKQTY") < 0) )
      throw new VetoException(res.getString("DM_Invalid_stock_qty"));
    if ( productDataSet.isNull("MINREORDERQTY") || (productDataSet.getInt("MINREORDERQTY") < 0) )
      throw new VetoException(res.getString("DM_Invalid_min_reorder_qty"));
    if ( productDataSet.isNull("ISACTIVE") || (productDataSet.getShort("ISACTIVE") < 0) )
      throw new VetoException(res.getString("DM_Invalid_active_value"));
  }

  /**
   * Do some row validation before adding new customer record
   * or before updating customer record.
   */
  private void validateCustomerRow() throws Exception {
    // In validating an customer record check the following:
    // - must have valid Name i.e. FirstName and LastName cannot both be blank
    if ( customerDataSet.isNull("FIRSTNAME") ||
        (customerDataSet.getString("FIRSTNAME").trim().compareTo("") == 0) )
      throw new VetoException(res.getString("DM_Invalid_first_name"));
    if ( customerDataSet.isNull("LASTNAME") ||
        (customerDataSet.getString("LASTNAME").trim().compareTo("") == 0) )
      throw new VetoException(res.getString("DM_Invalid_last_name"));
    if ( customerDataSet.isNull("PHONE") ||
        (customerDataSet.getString("PHONE").trim().compareTo("") == 0) )
      throw new VetoException(res.getString("DM_Invalid_phone"));
  }

  /**
   * Do some row validation before adding new category record
   * or before updating category record.
   */
  private void validateCategoryRow() throws Exception {
    // In validating an category record check the following:
    // - must have valid Category
    if ( categoryDataSet.isNull("CATEGORY") ||
        (categoryDataSet.getString("CATEGORY").trim().compareTo("") == 0) )
      //throw new DataSetException(DataSetException.FIELD_POST_ERROR,
      //                           res.getString("DM_Invalid_category_name"));
      throw new VetoException(res.getString("DM_Invalid_category_name"));
  }



  /**
   * When the STATUS column of the order is changed to "Fulfilled", then
   * set the ShipDate column to today's date.
   */
  void colOrdersStatus_changed(DataSet dataSet, Column column, Variant variant) throws DataSetException{
    if (variant.getString().compareTo(res.getString("DM_Fulfilled")) == 0) {
      // The SHIPDATE column is declared as type DATE in the Interbase
      // server as well as the persistent Column object colOrdersShipDate.
      // But we are getting the error "Unexpected type: TIMESTAMP Expected type:
      // DATE" if we use the setDate() method instead of the setTimestamp()
      // method:
      dataSet.setTimestamp("SHIPDATE", System.currentTimeMillis());
    }
  }

  /**
   * When the PayMethod column is changed to "CLIFFCARD", validate to make sure
   * that the customer has a valid Cliffhanger account in the accountDataSet.
   * If so, fill the CreditCard number.
   */
  void colOrdersPayMethod_validate(DataSet dataSet, Column column, Variant variant) throws Exception{
    if (variant.getString().compareTo(res.getString("DM_CLIFFCARD")) == 0) {
      // Lookup the CreditCardNumber for the current customer
      int customerID = dataSet.getInt("CUSTOMERID");
      if (customerID <= 0) {
        throw new VetoException(res.getString("DM_Invalid_customer"));
      }

      String creditCardNum = lookupCreditCardNumber(customerID);
      if (creditCardNum.compareTo("") == 0) {
        throw new VetoException(res.getString("DM_Non_Cliffcard_member"));
      }
      else {
        dataSet.setString("CREDITCARDNUM", creditCardNum);
      }
    }
  }

  /**
   * Method to lookup the CreditCardNumber given the customerID
   */
  private String lookupCreditCardNumber(int customerID) {
    String creditCardNum = new String("");
    try {
      // Define a DataRow to hold the customerID to look for
      // the Account Dataset, and another to hold the row
      // of account data that we find.
      accountDataSet.open();
      DataRow lookupRow = new DataRow(accountDataSet, "ID");
      DataRow accountRow = new DataRow(accountDataSet, new String[] {"ID", "CREDITCARDNUMBER"} );

      lookupRow.setInt("ID", customerID);

      if (accountDataSet.lookup(lookupRow, accountRow, Locate.FIRST)) {
        creditCardNum = accountRow.getString("CREDITCARDNUMBER");
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    };
    return creditCardNum;
  }

  /**
   * When the AmtPaid column changes, recalculate the aggregate totals for
   * the order items dataset.
   */
  void colOrdersAmtPaid_changed(DataSet dataSet, Column column, Variant variant) throws DataSetException{
    // To force the orderItemDataSet to recalculate the Aggregate columns, we
    // want to trigger the calcAggAdd event of the orderItemDataSet.
    // Do a dummy edit to the current record assigning the ID of the order
    // record to the OrderID column.
    if (orderItemDataSet.getRowCount() > 0) {
      orderItemDataSet.editRow();
      orderItemDataSet.setInt("ORDERID", dataSet.getInt("ID"));
      orderItemDataSet.post();
    }
  }

  void orderDataSet_modifying(DataSet dataSet) throws Exception{
    // Do not allow edits if the Order record
    // STATUS value is Fulfilled.
    String status = orderDataSet.getString("STATUS");
    if (status.compareTo(res.getString("DM_Fulfilled")) == 0) {
      // Show the message dialog here because there is
      // no error event handler for modifying event errors.
      CliffhangerApplication.messageDlg(
        res.getString("DM_Error"),
        res.getString("DM_Cannot_modify_order"),
        Message.OK);
      throw new VetoException(res.getString("DM_Cannot_modify_order"));
    }
  }
}

class DataModule1_orderItemDataSet_calcFieldsAdapter implements borland.jbcl.dataset.CalcFieldsListener{
  DataModule1 adaptee;

  DataModule1_orderItemDataSet_calcFieldsAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void calcFields(ReadRow readRow, DataRow dataRow, boolean boolean1) throws DataSetException, DataSetException{
    adaptee.orderItemDataSet_calcFields(readRow, dataRow, boolean1);
  }
}

class DataModule1_orderItemDataSet_calcAggFieldsAdapter extends borland.jbcl.dataset.CalcAggFieldsAdapter {
  DataModule1 adaptee;

  DataModule1_orderItemDataSet_calcAggFieldsAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void calcAggAdd(ReadRow readRow, ReadWriteRow readWriteRow) throws DataSetException, DataSetException{
    adaptee.orderItemDataSet_calcAggAdd(readRow, readWriteRow);
  }

  public void calcAggDelete(ReadRow readRow, ReadWriteRow readWriteRow) throws DataSetException{
    adaptee.orderItemDataSet_calcAggDelete(readRow, readWriteRow);
  }
}

class DataModule1_orderDataSet_calcFieldsAdapter implements borland.jbcl.dataset.CalcFieldsListener{
  DataModule1 adaptee;

  DataModule1_orderDataSet_calcFieldsAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void calcFields(ReadRow readRow, DataRow dataRow, boolean boolean1) throws DataSetException, DataSetException{
    adaptee.orderDataSet_calcFields(readRow, dataRow, boolean1);
  }
}

class DataModule1_colOrderItemProductID_columnChangeAdapter extends borland.jbcl.dataset.ColumnChangeAdapter {
  DataModule1 adaptee;

  DataModule1_colOrderItemProductID_columnChangeAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void changed(DataSet dataSet, Column column, Variant variant) throws DataSetException{
    adaptee.colOrderItemProductID_changed(dataSet, column, variant);
  }
}

class DataModule1_colOrdersCustomerID_columnChangeAdapter extends borland.jbcl.dataset.ColumnChangeAdapter {
  DataModule1 adaptee;

  DataModule1_colOrdersCustomerID_columnChangeAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void changed(DataSet dataSet, Column column, Variant variant) throws DataSetException{
    adaptee.colOrdersCustomerID_changed(dataSet, column, variant);
  }
}

class DataModule1_orderItemDataSet_editAdapter extends borland.jbcl.dataset.EditAdapter {
  DataModule1 adaptee;

  DataModule1_orderItemDataSet_editAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void adding(DataSet dataSet, ReadWriteRow readWriteRow) throws Exception{
    adaptee.orderItemDataSet_adding(dataSet, readWriteRow);
  }

  public void updating(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow) throws Exception{
    adaptee.orderItemDataSet_updating(dataSet, readWriteRow, readRow);
  }

  public void deleting(DataSet dataSet) throws Exception{
    adaptee.orderItemDataSet_deleting(dataSet);
  }

  public void modifying(DataSet dataSet) throws Exception{
    adaptee.orderItemDataSet_modifying(dataSet);
  }

  public void inserting(DataSet dataSet) throws Exception{
    adaptee.orderItemDataSet_inserting(dataSet);
  }

  public void canceling(DataSet dataSet) throws Exception{
    adaptee.orderItemDataSet_canceling(dataSet);
  }

  public void editError(DataSet dataSet, Column column, Variant variant, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.orderItemDataSet_editError(dataSet, column, variant, dataSetException, errorResponse);
  }

  public void addError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.orderItemDataSet_addError(dataSet, readWriteRow, dataSetException, errorResponse);
  }

  public void updateError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.orderItemDataSet_updateError(dataSet, readWriteRow, dataSetException, errorResponse);
  }
}

class DataModule1_resProduct_resolverAdapter extends borland.jbcl.dataset.ResolverAdapter {
  DataModule1 adaptee;

  DataModule1_resProduct_resolverAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void updateError(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow, ReadWriteRow readWriteRow1, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.resProduct_updateError(dataSet, readWriteRow, readRow, readWriteRow1, dataSetException, errorResponse);
  }
}

class DataModule1_customerDataSet_editAdapter extends borland.jbcl.dataset.EditAdapter {
  DataModule1 adaptee;

  DataModule1_customerDataSet_editAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void adding(DataSet dataSet, ReadWriteRow readWriteRow) throws Exception{
    adaptee.customerDataSet_adding(dataSet, readWriteRow);
  }

  public void deleting(DataSet dataSet) throws Exception{
    adaptee.customerDataSet_deleting(dataSet);
  }

  public void added(DataSet dataSet) throws DataSetException{
    adaptee.customerDataSet_added(dataSet);
  }

  public void updated(DataSet dataSet) throws DataSetException{
    adaptee.customerDataSet_updated(dataSet);
  }

  public void deleted(DataSet dataSet) throws DataSetException{
    adaptee.customerDataSet_deleted(dataSet);
  }

  public void updating(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow) throws Exception{
    adaptee.customerDataSet_updating(dataSet, readWriteRow, readRow);
  }

  public void canceling(DataSet dataSet) throws Exception{
    adaptee.customerDataSet_canceling(dataSet);
  }

  public void editError(DataSet dataSet, Column column, Variant variant, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.customerDataSet_editError(dataSet, column, variant, dataSetException, errorResponse);
  }

  public void addError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.customerDataSet_addError(dataSet, readWriteRow, dataSetException, errorResponse);
  }

  public void updateError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.customerDataSet_updateError(dataSet, readWriteRow, dataSetException, errorResponse);
  }
}

class DataModule1_orderDataSet_editAdapter extends borland.jbcl.dataset.EditAdapter {
  DataModule1 adaptee;

  DataModule1_orderDataSet_editAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void deleting(DataSet dataSet) throws Exception{
    adaptee.orderDataSet_deleting(dataSet);
  }

  public void adding(DataSet dataSet, ReadWriteRow readWriteRow) throws Exception{
    adaptee.orderDataSet_adding(dataSet, readWriteRow);
  }

  public void updating(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow) throws Exception{
    adaptee.orderDataSet_updating(dataSet, readWriteRow, readRow);
  }

  public void canceling(DataSet dataSet) throws Exception{
    adaptee.orderDataSet_canceling(dataSet);
  }

  public void editError(DataSet dataSet, Column column, Variant variant, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.orderDataSet_editError(dataSet, column, variant, dataSetException, errorResponse);
  }

  public void addError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.orderDataSet_addError(dataSet, readWriteRow, dataSetException, errorResponse);
  }

  public void updateError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.orderDataSet_updateError(dataSet, readWriteRow, dataSetException, errorResponse);
  }

  public void modifying(DataSet dataSet) throws Exception{
    adaptee.orderDataSet_modifying(dataSet);
  }
}

class DataModule1_categoryDataSet_editAdapter extends borland.jbcl.dataset.EditAdapter {
  DataModule1 adaptee;

  DataModule1_categoryDataSet_editAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void deleting(DataSet dataSet) throws Exception{
    adaptee.categoryDataSet_deleting(dataSet);
  }

  public void deleted(DataSet dataSet) throws DataSetException{
    adaptee.categoryDataSet_deleted(dataSet);
  }

  public void added(DataSet dataSet) throws DataSetException{
    adaptee.categoryDataSet_added(dataSet);
  }

  public void updated(DataSet dataSet) throws DataSetException{
    adaptee.categoryDataSet_updated(dataSet);
  }

  public void adding(DataSet dataSet, ReadWriteRow readWriteRow) throws Exception{
    adaptee.categoryDataSet_adding(dataSet, readWriteRow);
  }

  public void canceling(DataSet dataSet) throws Exception{
    adaptee.categoryDataSet_canceling(dataSet);
  }

  public void updating(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow) throws Exception{
    adaptee.categoryDataSet_updating(dataSet, readWriteRow, readRow);
  }

  public void editError(DataSet dataSet, Column column, Variant variant, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.categoryDataSet_editError(dataSet, column, variant, dataSetException, errorResponse);
  }

  public void addError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.categoryDataSet_addError(dataSet, readWriteRow, dataSetException, errorResponse);
  }

  public void updateError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.categoryDataSet_updateError(dataSet, readWriteRow, dataSetException, errorResponse);
  }
}

class DataModule1_colOrdersStatus_columnChangeAdapter extends borland.jbcl.dataset.ColumnChangeAdapter {
  DataModule1 adaptee;

  DataModule1_colOrdersStatus_columnChangeAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void changed(DataSet dataSet, Column column, Variant variant) throws DataSetException{
    adaptee.colOrdersStatus_changed(dataSet, column, variant);
  }
}

class DataModule1_productDataSet_editAdapter extends borland.jbcl.dataset.EditAdapter {
  DataModule1 adaptee;

  DataModule1_productDataSet_editAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void deleting(DataSet dataSet) throws Exception{
    adaptee.productDataSet_deleting(dataSet);
  }

  public void added(DataSet dataSet) throws DataSetException{
    adaptee.productDataSet_added(dataSet);
  }

  public void updated(DataSet dataSet) throws DataSetException{
    adaptee.productDataSet_updated(dataSet);
  }

  public void deleted(DataSet dataSet) throws DataSetException{
    adaptee.productDataSet_deleted(dataSet);
  }

  public void adding(DataSet dataSet, ReadWriteRow readWriteRow) throws Exception{
    adaptee.productDataSet_adding(dataSet, readWriteRow);
  }

  public void updating(DataSet dataSet, ReadWriteRow readWriteRow, ReadRow readRow) throws Exception{
    adaptee.productDataSet_updating(dataSet, readWriteRow, readRow);
  }

  public void canceling(DataSet dataSet) throws Exception{
    adaptee.productDataSet_canceling(dataSet);
  }

  public void editError(DataSet dataSet, Column column, Variant variant, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.productDataSet_editError(dataSet, column, variant, dataSetException, errorResponse);
  }

  public void addError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.productDataSet_addError(dataSet, readWriteRow, dataSetException, errorResponse);
  }

  public void updateError(DataSet dataSet, ReadWriteRow readWriteRow, DataSetException dataSetException, ErrorResponse errorResponse) throws DataSetException{
    adaptee.productDataSet_updateError(dataSet, readWriteRow, dataSetException, errorResponse);
  }
}

class DataModule1_colOrdersPayMethod_columnChangeAdapter extends borland.jbcl.dataset.ColumnChangeAdapter {
  DataModule1 adaptee;

  DataModule1_colOrdersPayMethod_columnChangeAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void validate(DataSet dataSet, Column column, Variant variant) throws Exception{
    adaptee.colOrdersPayMethod_validate(dataSet, column, variant);
  }
}

class DataModule1_colOrdersAmtPaid_columnChangeAdapter extends borland.jbcl.dataset.ColumnChangeAdapter {
  DataModule1 adaptee;

  DataModule1_colOrdersAmtPaid_columnChangeAdapter(DataModule1 adaptee) {
    this.adaptee = adaptee;
  }

  public void changed(DataSet dataSet, Column column, Variant variant) throws DataSetException{
    adaptee.colOrdersAmtPaid_changed(dataSet, column, variant);
  }
}
